/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.command;

import java.time.ZonedDateTime;
import java.util.Map;

import org.springframework.data.annotation.Id;
import org.springframework.data.annotation.TypeAlias;
import org.springframework.data.mongodb.core.mapping.Document;

import de.ozgcloud.command.Command;
import de.ozgcloud.command.CommandStatus;
import de.ozgcloud.vorgang.vorgang.redirect.RedirectRequest;
import lombok.Builder;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;

@ToString
@Getter
@Builder
@Document(collection = Command.COLLECTION_NAME)
@TypeAlias("Command")
public class PersistedCommand implements Command {

	public static final String ID_FIELD = "id";
	public static final String VORGANG_ID_FIELD = "vorgangId";
	public static final String RELATION_ID_FIELD = "relationId";
	public static final String RELATION_VERSION_FIELD = "relationVersion";
	public static final String ORDER_FIELD = "order";

	public static final String CREATED_AT_FIELD = "createdAt";
	public static final String FINISHED_AT_FIELD = "finishedAt";

	public static final String CREATED_BY_FIELD = "createdBy";

	public static final String BODY_OBJECT_FIELD = "bodyObject";

	public static final String ERROR_MSG_FIELD = "errorMessage";

	public static final String PREVIOUS_STATE_FIELD = "previousState";
	public static final String CREATED_RESOURCE_FIELD = "createdResource";

	static final String PROPERTY_PARENT_ID = "parentId";
	static final String PROPERTY_EXECUTION_MODE = "executionMode";
	static final String PROPERTY_COMPLETE_IF_SUBS_COMPLETED = "completedIfSubsCompleted";

	@Id
	private String id;
	private String vorgangId;

	private ZonedDateTime createdAt;
	private ZonedDateTime finishedAt;

	private String createdBy;
	private String createdByName;

	@Setter
	private CommandStatus status;

	private String relationId;
	private Long relationVersion;

	private String order;

	private String errorMessage;

	private RedirectRequest redirectRequest;

	private Map<String, String> body;
	private Map<String, Object> bodyObject;

	@Setter
	private Map<String, Object> previousState;

	private String createdResource;
}
