package de.ozgcloud.vorgang.common.db;

import org.springframework.data.mongodb.core.aggregation.Aggregation;
import org.springframework.data.mongodb.core.aggregation.ComparisonOperators.Eq;
import org.springframework.data.mongodb.core.aggregation.ConditionalOperators.Switch;
import org.springframework.data.mongodb.core.aggregation.ConditionalOperators.Switch.CaseOperator;
import org.springframework.data.mongodb.core.aggregation.SetOperation;
import org.springframework.data.mongodb.core.query.UpdateDefinition;

import de.ozgcloud.command.CommandStatus;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
public class MongoDbUtil {

	public static final String MONGODB_STATUS = "status";

	private static final String MONGODB_REFERENCE_STATUS = "$status";

	public static UpdateDefinition buildConditionalStatusUpdateDefinition(CaseOperator... conditions) {
		return Aggregation.newUpdate(SetOperation.set(MONGODB_STATUS).toValue(createStatusSwitch(conditions)));
	}

	private static Switch createStatusSwitch(CaseOperator... conditions) {
		return createSwitch(MONGODB_REFERENCE_STATUS, conditions);
	}

	private static Switch createSwitch(String field, CaseOperator... conditions) {
		return Switch.switchCases(conditions).defaultTo(field);
	}

	public static CaseOperator createStatusCaseOperator(CommandStatus beEqualWith, CommandStatus shouldSetTo) {
		return createCaseOperator(MONGODB_STATUS, beEqualWith, shouldSetTo);
	}

	private static CaseOperator createCaseOperator(String field, Object beEqualWith, Object shouldSetTo) {
		return CaseOperator.when(Eq.valueOf(field).equalToValue(beEqualWith)).then(shouldSetTo);
	}
}
