package de.ozgcloud.vorgang.registry;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.vorgang.callcontext.VorgangManagerClientCallContextAttachingInterceptor;
import de.ozgcloud.zufi.grpc.registration.GrpcVorgangManagerRegistrationRequest;
import de.ozgcloud.zufi.grpc.registration.VorgangManagerRegistrationServiceGrpc.VorgangManagerRegistrationServiceBlockingStub;

class ZufiRemoteServiceTest {
	private static final String ORGANISATIONS_EINHEITEN_ID = "123456";
	private static final List<String> ORGANISATIONS_EINHEITEN_IDS = List.of(ORGANISATIONS_EINHEITEN_ID);
	private static final String VORGANG_MANAGER_ADDRESS = "address";

	@Spy
	@InjectMocks
	private ZufiRemoteService zufiRemoteService;

	@Mock
	private VorgangManagerRegistrationServiceBlockingStub serviceStub;

	@DisplayName("Test ZuFiRemoteService when VorgangManager address is configured")
	@Nested
	class TestRegisteringVorgangManager {
		@BeforeEach
		void init() {
			when(serviceStub.withInterceptors(any())).thenReturn(serviceStub);
			when(serviceStub.register(any(GrpcVorgangManagerRegistrationRequest.class))).thenReturn(GrpcRegistrationResponseTestFactory.create());
		}

		@Test
		void shouldCallServiceStub() {
			zufiRemoteService.registerVorgangManager(ORGANISATIONS_EINHEITEN_IDS, VORGANG_MANAGER_ADDRESS);

			verify(serviceStub).register(any(GrpcVorgangManagerRegistrationRequest.class));
		}

		@Test
		void shouldUseInterceptor() {
			zufiRemoteService.registerVorgangManager(ORGANISATIONS_EINHEITEN_IDS, VORGANG_MANAGER_ADDRESS);

			verify(serviceStub).withInterceptors(any(VorgangManagerClientCallContextAttachingInterceptor.class));
		}
	}
}
