/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.system;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.boot.info.BuildProperties;
import org.springframework.test.util.ReflectionTestUtils;

import de.ozgcloud.vorgang.common.search.SearchService;
import io.grpc.stub.StreamObserver;

class SystemStatusGrpcServiceTest {

	@InjectMocks
	private SystemStatusGrpcService systemStatusService;

	@Mock
	private SearchService searchService;

	@Mock
	private BuildProperties buildProperties;

	@Nested
	@DisplayName("Request system status")
	class TestGetSystemStatus {

		@Mock
		private StreamObserver<GrpcGetSystemStatusResponse> responseObserver;
		@Captor
		private ArgumentCaptor<GrpcGetSystemStatusResponse> responseCaptor;

		private static final String BUILD_VERSION = "b1";
		private static final String JAVA_VERSION_PROPERTY = "java.version";

		@BeforeEach
		void init() {
			when(buildProperties.getVersion()).thenReturn(BUILD_VERSION);
		}

		@Test
		void shouldResponseWithSearchServerStatus() {
			getSystemStatus();

			assertThat(responseCaptor.getValue().getIsSearchServerAvailable()).isTrue();
		}

		@Test
		void shouldResponseFalseWhenSearchServerNotConfigured() {
			ReflectionTestUtils.setField(systemStatusService, "searchService", null);

			getSystemStatus();

			assertThat(responseCaptor.getValue().getIsSearchServerAvailable()).isFalse();
		}

		@Test
		void shouldResponseWithJavaVersion() {
			getSystemStatus();

			assertThat(responseCaptor.getValue().getJavaVersion()).isEqualTo(System.getProperty(JAVA_VERSION_PROPERTY));
		}

		@Test
		void shouldResponseWithBuildVersion() {
			getSystemStatus();

			assertThat(responseCaptor.getValue().getBuildVersion()).isEqualTo(BUILD_VERSION);
		}

		@Test
		void shouldCallComplete() {
			getSystemStatus();

			verify(responseObserver, times(1)).onCompleted();
		}

		private void getSystemStatus() {
			systemStatusService.getSystemStatus(GrpcGetSystemStatusRequest.newBuilder().build(), responseObserver);

			verify(responseObserver).onNext(responseCaptor.capture());
		}
	}

}